# -*- encoding: utf-8 -*-

from datetime import datetime, date, time
from decimal import Decimal

SECRET_KEY = 'cheese'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
    }
}

INSTALLED_APPS = (
    'django.contrib.sessions',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.admin',

    'constance',
    'constance.backends.database',

    'testproject.test_app',
)

ROOT_URLCONF = 'testproject.urls'

CONSTANCE_CONNECTION_CLASS = 'testproject.test_app.redis_mockup.Connection'

CONSTANCE_CONFIG = {
    'INT_VALUE': {
        'default': 1,
        'help_text': 'some int',
    },
    'LONG_VALUE': {
        'default': 123456L,
        'help_text': 'some looong int',
    },
    'BOOL_VALUE': {
        'default': True,
        'help_text': 'true or false',
    },
    'STRING_VALUE_REQUIRED': {
        'default': 'Hello world',
        'help_text': 'greetings',
    },
    'STRING_VALUE_NOT_REQUIRED': {
        'default': 'Hello world',
        'help_text': 'greetings',
        'required': False,
    },
    'UNICODE_VALUE': {
        'default': 'Rivière-Bonjour'.decode('utf-8'),
        'help_text': 'greetings',
    },
    'DECIMAL_VALUE': {
        'default': Decimal('0.1'),
        'help_text': 'the first release version',
    },
    'DATETIME_VALUE': {
        'default': datetime(2010, 8, 23, 11, 29, 24),
        'help_text': 'time of the first commit',
    },
    'FLOAT_VALUE': {
        'default': 3.1415926536,
        'help_text': 'PI',
    },
    'DATE_VALUE': {
        'default': date(2010, 12, 24),
        'help_text': 'Merry Chrismas',
    },
    'TIME_VALUE': {
        'default': time(23, 59, 59),
        'help_text': 'And happy New Year',
    },
    'COMPAT_INT_VALUE': (1, 'some int'),
    'COMPAT_LONG_VALUE': (123456L, 'some looong int'),
    'COMPAT_BOOL_VALUE': (True, 'true or false'),
    'COMPAT_STRING_VALUE': ('Hello world', 'greetings'),
    'COMPAT_UNICODE_VALUE': ('Rivière-Bonjour'.decode('utf-8'), 'greetings'),
    'COMPAT_DECIMAL_VALUE': (Decimal('0.1'), 'the first release version'),
    'COMPAT_DATETIME_VALUE': (datetime(2010, 8, 23, 11, 29, 24), 'time of the first commit'),
    'COMPAT_FLOAT_VALUE': (3.1415926536, 'PI'),
    'COMPAT_DATE_VALUE': (date(2010, 12, 24),  'Merry Chrismas'),
    'COMPAT_TIME_VALUE': (time(23, 59, 59),  'And happy New Year'),
}



from django.core.exceptions import ImproperlyConfigured
from django.utils.importlib import import_module

CONFIGURATION_ERROR = ("Constance configuration key '%s' must have either a "
                       "2-tuple with default value and help text or a dictionary "
                       "with at least the 'default' and 'help_text' set.")

def import_module_attr(path):
    package, module = path.rsplit('.', 1)
    return getattr(import_module(package), module)

def parse_config(config):
    """ Parse Constance configuration dictionary

    Checks if mandatory keywords are set and provides backwards compatibility
    for (default, help_text) tuple based CONSTANCE_CONFIG.
    """
    if not config:
        return config

    cfg = {}
    for key, value in config.iteritems():
        if isinstance(value, (tuple, list)):
            if len(value) == 2:
                default, help_text = value
                cfg[key] = {
                    'default': default,
                    'help_text': help_text,
                }
                continue
        elif isinstance(value, dict):
            if 'default' in value and 'help_text' in value:
                cfg[key] = value
                continue
        raise ImproperlyConfigured(CONFIGURATION_ERROR % (key, ))
    return cfg



# -*- encoding: utf-8 -*-

import sys
from datetime import datetime, date, time
from decimal import Decimal

from django.test import TestCase
from django.conf import settings
from django.contrib import admin
from django.contrib.auth.models import User

from constance import settings
from constance.admin import Config

# Use django RequestFactory later on
from testproject.test_app.tests.helpers import FakeRequest


class TestStorage(object):

    def test_store(self):
        # read defaults
        del sys.modules['constance']
        from constance import config
        self.assertEquals(config.INT_VALUE, 1)
        self.assertEquals(config.LONG_VALUE, 123456L)
        self.assertEquals(config.BOOL_VALUE, True)
        self.assertEquals(config.STRING_VALUE_REQUIRED, 'Hello world')
        self.assertEquals(config.UNICODE_VALUE, u'Rivi\xe8re-Bonjour')
        self.assertEquals(config.DECIMAL_VALUE, Decimal('0.1'))
        self.assertEquals(config.DATETIME_VALUE, datetime(2010, 8, 23, 11, 29, 24))
        self.assertEquals(config.FLOAT_VALUE, 3.1415926536)
        self.assertEquals(config.DATE_VALUE, date(2010, 12, 24))
        self.assertEquals(config.TIME_VALUE, time(23, 59, 59))

        # set values
        config.INT_VALUE = 100
        config.LONG_VALUE = 654321L
        config.BOOL_VALUE = False
        config.STRING_VALUE_REQUIRED = 'Beware the weeping angel'
        config.UNICODE_VALUE = 'Québec'.decode('utf-8')
        config.DECIMAL_VALUE = Decimal('1.2')
        config.DATETIME_VALUE = datetime(1977, 10, 2)
        config.FLOAT_VALUE = 2.718281845905
        config.DATE_VALUE = date(2001, 12, 20)
        config.TIME_VALUE = time(1, 59, 0)

        # read again
        self.assertEquals(config.INT_VALUE, 100)
        self.assertEquals(config.LONG_VALUE, 654321L)
        self.assertEquals(config.BOOL_VALUE, False)
        self.assertEquals(config.STRING_VALUE_REQUIRED, 'Beware the weeping angel')
        self.assertEquals(config.UNICODE_VALUE, 'Québec'.decode('utf-8'))
        self.assertEquals(config.DECIMAL_VALUE, Decimal('1.2'))
        self.assertEquals(config.DATETIME_VALUE, datetime(1977, 10, 2))
        self.assertEquals(config.FLOAT_VALUE, 2.718281845905)
        self.assertEquals(config.DATE_VALUE, date(2001, 12, 20))
        self.assertEquals(config.TIME_VALUE, time(1, 59, 0))

    def test_nonexistent(self):
        from constance import config
        try:
            config.NON_EXISTENT
        except Exception, e:
            pass
        self.assertEquals(type(e), AttributeError)

        try:
            config.NON_EXISTENT = 1
        except Exception, e:
            pass
        self.assertEquals(type(e), AttributeError)

class TestRedis(TestCase, TestStorage):

    def setUp(self):
        self.old_backend = settings.BACKEND
        settings.BACKEND = 'constance.backends.redisd.RedisBackend'
        del sys.modules['constance']
        from constance import config
        config._backend._rd.clear()

    def tearDown(self):
        del sys.modules['constance']
        from constance import config
        config._backend._rd.clear()
        settings.BACKEND = self.old_backend
        import constance
        constance.config = Config()

    def testMissingValues(self):
        from constance import config

        # set some values and leave out others
        config.LONG_VALUE = 654321L
        config.BOOL_VALUE = False
        config.UNICODE_VALUE = 'Québec'.decode('utf-8')
        config.DECIMAL_VALUE = Decimal('1.2')
        config.DATETIME_VALUE = datetime(1977, 10, 2)
        config.DATE_VALUE = date(2001, 12, 20)
        config.TIME_VALUE = time(1, 59, 0)

        self.assertEquals(config.INT_VALUE, 1)  # this should be the default value
        self.assertEquals(config.LONG_VALUE, 654321L)
        self.assertEquals(config.BOOL_VALUE, False)
        self.assertEquals(config.STRING_VALUE_REQUIRED, 'Hello world')  # this should be the default value
        self.assertEquals(config.UNICODE_VALUE, 'Québec'.decode('utf-8'))
        self.assertEquals(config.DECIMAL_VALUE, Decimal('1.2'))
        self.assertEquals(config.DATETIME_VALUE, datetime(1977, 10, 2))
        self.assertEquals(config.FLOAT_VALUE, 3.1415926536)  # this should be the default value
        self.assertEquals(config.DATE_VALUE, date(2001, 12, 20))
        self.assertEquals(config.TIME_VALUE, time(1, 59, 0))


class TestDatabase(TestCase, TestStorage):

    def setUp(self):
        self.old_backend = settings.BACKEND
        settings.BACKEND = 'constance.backends.database.DatabaseBackend'

    def tearDown(self):
        del sys.modules['constance']
        settings.BACKEND = self.old_backend
        import constance
        constance.config = Config()

class TestAdmin(TestCase):
    model = Config

    def setUp(self):
        self.user = User.objects.create_superuser('admin', 'nimda', 'a@a.cz')
        self.options = admin.site._registry[self.model]
        self.fake_request = FakeRequest(user=self.user)
        self.client.login(username=self.user, password='nimda')

    def test_changelist(self):
        response = self.options.changelist_view(self.fake_request, {})
        self.assertEquals(response.status_code, 200)




import datetime

from django.core.handlers.wsgi import WSGIRequest



class FakeRequest(WSGIRequest):

    def __init__(self, user=None, meta=None, environ=None, cookies=None):
        if not environ:
            environ = {
                'PATH_INFO':         '/',
                'QUERY_STRING':      '',
                'REMOTE_ADDR':       '127.0.0.1',
                'REQUEST_METHOD':    'GET',
                'SCRIPT_NAME':       '',
                'SERVER_NAME':       'testserver',
                'SERVER_PORT':       '80',
                'SERVER_PROTOCOL':   'HTTP/1.1',
                'wsgi.version':      (1,0),
                'wsgi.url_scheme':   'http',
                'wsgi.errors':       [],
                'wsgi.multiprocess': True,
                'wsgi.multithread':  False,
                'wsgi.run_once':     False,
                'wsgi.input':        None,
            }
        super(FakeRequest, self).__init__(environ)
        if user:
            self.user = user
        if meta:
            self.META.update(meta)
        if cookies:
            self.COOKIES.update(cookies)
        self.xnow = datetime.datetime.now()

    @classmethod
    def from_test_response(cls, response, *args, **kwargs):
        return cls(environ=response.request, *args, **kwargs)





from django.contrib import admin

from django.conf.urls.defaults import patterns, include

urlpatterns = patterns('',
    # Uncomment the admin/doc line below and add 'django.contrib.admindocs'
    # to INSTALLED_APPS to enable admin documentation:
    # (r'^admin/doc/', include('django.contrib.admindocs.urls')),

    (r'^admin/', include(admin.site.urls)),
)

